/**
 * MessagesFrame.java
 *
 * A Tree Exerciser
 *
 */

import java.awt.event.*;
import javax.swing.*;

import java.awt.BorderLayout;
import java.awt.Component;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeSelectionModel;

/**
 * Frame class.
 */
public class MessagesFrame extends JFrame
{
	
	MessageBox messagebox=new MessageBox();
	
	JTree messagetree;
	
	public MessagesFrame()
	{
		super("Messages");
		
		getContentPane().setLayout(new BorderLayout());
		
		JPanel buttonpanel=new JPanel();
		
		JButton add1000button = new JButton("Add 1000 Messages");
		JButton addonebutton= new JButton("Add One Message");
		JButton expandbutton = new JButton("Expand All");
		JButton deletebutton = new JButton("Delete Selected");
		
		buttonpanel.add(add1000button);
		buttonpanel.add(addonebutton);
		buttonpanel.add(expandbutton);
		buttonpanel.add(deletebutton);
		
		this.getContentPane().add(BorderLayout.NORTH,buttonpanel);
		
		messagetree=new JTree(new MessageBoxTreeModel(messagebox));
		messagetree.setRootVisible(false);
		messagetree.setShowsRootHandles(true);
		messagetree.setCellRenderer(new MessageRenderer());
		messagetree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		
		messagetree.setRowHeight(16);
		messagetree.setLargeModel(true);
		
		this.getContentPane().add(BorderLayout.CENTER,new JScrollPane(messagetree));
		
		this.addWindowListener
		(
			new WindowAdapter()
			{
				public void windowClosing(WindowEvent e)
				{
					MessagesFrame.this.windowClosed();
				}
			}
		);
		
		add1000button.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MessagesFrame.this.populate();
				}
			}
		);
		
		addonebutton.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MessagesFrame.this.addone();
				}
			}
		);
		
		expandbutton.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MessagesFrame.this.expandRows();
				}
			}
		);
		
		deletebutton.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MessagesFrame.this.deleteSelected();
				}
			}
		);
	}
	
	private void deleteSelected()
	{
		final Long selectedid=(Long)messagetree.getSelectionPath().getLastPathComponent();
		
		if(selectedid==null) return;
		
		Runnable rb=new Runnable()
		{
			public void run()
			{
				messagebox.deleteMessage(selectedid);
			}
		};
		
		Thread t=new Thread(rb);
		t.start();
	}
	
	private void populate()
	{
		Runnable rb=new Runnable()
		{
			public void run()
			{
				System.out.println("Thread started to add 1000");
				for(int i=0;i<1000; i++)
				{
					messagebox.generateMessage();
				}
				System.out.println("Thread completed");
			}};
		
		Thread t=new Thread(rb);
		t.start();
	}
	
	private void addone()
	{
		messagebox.generateMessage();
	}
	
	private void expandRows()
	{
		int row=0;
		while (row < messagetree.getRowCount())
		{
			messagetree.expandRow(row);
			row++;
		}
	}
	
	protected void windowClosed()
	{
		System.exit(0);
	}
	
	class MessageRenderer extends DefaultTreeCellRenderer
	{
		public Component getTreeCellRendererComponent(
			JTree tree,
			Object value,
			boolean sel,
			boolean expanded,
			boolean leaf,
			int row,
			boolean hasFocus)
		{
			
			super.getTreeCellRendererComponent(
				tree, value, sel,
				expanded, leaf, row,
				hasFocus);
			
			if(value instanceof Long)
			{
				setText(messagebox.getMessageAsString((Long)value));
			}
			
			return this;
		}
	}
}
